@@ -17,10 +17,11 @@ architectures, or licensing concerns.
 
 """
 
-__version__ = '0.3.2'
+__version__ = "0.4.3"
 
 from pisces.util import db_connect, get_tables, travel_times, make_table
 from pisces.schema.util import string_formatter
 from pisces.io.trace import wfdisc2trace
 from pisces.schema import kbcore
 from pisces.io.readwaveform import read_waveform
+from pisces import request
