@@ -1,7 +1,5 @@
 """
-request.py
-
-Convenience functions for common queries.
+Convenience functions for building common queries.
 
 """
 import numpy as np
@@ -10,7 +8,9 @@ from obspy.core import UTCDateTime, Stream
 import obspy.geodetics as geod
 
 from pisces.io.trace import wfdisc2trace
+from pisces.util import make_wildcard_list
 
+import warnings
 
 def get_wfdisc_rows(session, wfdisc, sta=None, chan=None, t1=None, t2=None,
                     wfids=None, daylong=False, asquery=False, verbose=False):
@@ -51,9 +51,11 @@ def get_wfdisc_rows(session, wfdisc, sta=None, chan=None, t1=None, t2=None,
         q = q.filter(wfdisc.wfid.in_(wfids))
     else:
         if sta is not None:
-            q = q.filter(wfdisc.sta == sta)
+            sta = make_wildcard_list(sta)
+            q = q.filter(or_(*[wfdisc.sta.like(stas) for stas in sta]))
         if chan is not None:
-            q = q.filter(wfdisc.chan == chan)
+            chan = make_wildcard_list(chan)
+            q = q.filter(or_(*[wfdisc.chan.like(chans) for chans in chan]))
         if [t1, t2].count(None) == 0:
             q = q.filter(wfdisc.time.between(t1 - CHUNKSIZE, t2))
             q = q.filter(wfdisc.endtime > t1)
@@ -100,7 +102,7 @@ def distaz_query(records, deg=None, km=None, swath=None):
 
     """
     #initial True array to propagate through multiple logical AND comparisons
-    mask0 = np.ones(len(records), dtype=np.bool)
+    mask0 = np.ones(len(records), dtype=bool)
 
     if deg:
         dgen = (geod.locations2degrees(irec.lat, irec.lon, deg[0], deg[1]) \
@@ -293,7 +295,7 @@ def get_events(session, origin, event=None, region=None, deg=None, km=None,
 
 def get_stations(session, site, sitechan=None, affiliation=None, stations=None, 
         channels=None, nets=None, loc=None, region=None, deg=None, km=None, 
-        swath=None, stime=None, asquery=False):
+        swath=None, time_span=None, asquery=False):
     """
     Build common queries for stations.
 
@@ -328,6 +330,13 @@ def get_stations(session, site, sitechan=None, affiliation=None, stations=None,
         Useful if additional you desire additional sorting of filtering, or
         if you have your own in-database geographic query function(s).  If 
         supplied, deg, km, and/or swath are ignored in the returned query.
+    time_span : tuple or list
+        (startdate, enddate) or [startdate, enddate]
+        startdate and enddate are integer julian days of when you want stations (YYYYddd).  
+        If stations were moved one or more times in the time_span, you will 
+        get multiple copies of the station with updated gps values. If you want to be 
+        guaranteed a specific station at a specific time, startdate and enddate must 
+        both be included, even if they are the same.
 
     Notes
     -----
@@ -360,28 +369,30 @@ def get_stations(session, site, sitechan=None, affiliation=None, stations=None,
     Affiliation = affiliation
 
     d = deg
-    t = stime
-
+    
     q = session.query(Site)
-
+    
     if stations:
-        q = q.filter(Site.sta.in_(stations))
-
+        stations = make_wildcard_list(stations)
+        q = q.filter(or_(*[Site.sta.like(stas) for stas in stations]))
+        
     if nets:
+        nets = make_wildcard_list(nets)
         q = q.join(Affiliation, Affiliation.sta==Site.sta)
-        if isinstance(nets, list):
-            q = q.filter(Affiliation.net.in_(nets))
-        else:
-            q = q.filter(func.regexp_like(Affiliation.net, nets))
+        q = q.filter(or_(*[Affiliation.net.like(net) for net in nets]))
 
     if channels:
+        channels = make_wildcard_list(channels)
         q = q.join(Sitechan, Sitechan.sta==Site.sta)
-        if isinstance(channels, str):
-            #interpret string as regexp
-            q = q.filter(func.regexp_like(Sitechan.chan, channnels))
-        else:
-            q = q.filter(Sitechan.chan.in_(channels))
-    
+        q = q.filter(or_(*[Sitechan.chan.like(chans) for chans in channels]))
+
+    if time_span:
+        start_date, end_date = time_span  # start and end days of time period to get stations from
+        if start_date is not None:
+            q = q.filter(Site.ondate <= start_date)
+        if end_date is not None:
+            q = q.filter(Site.offdate >= end_date)
+
     q = geographic_query(q, Site, region=region, asquery=True)
 
     if asquery:
@@ -436,13 +447,16 @@ def get_arrivals(session, arrival, assoc=None, stations=None, channels=None,
     q = session.query(Arrival)
 
     if stations:
-        q = q.filter(Arrival.sta.in_(stations))
+        stations = make_wildcard_list(stations)
+        q = q.filter(or_(*[Arrival.sta.like(stas) for stas in stations]))
 
     if channels:
-        q = q.filter(Arrival.chan.in_(channels))
+        channels = make_wildcard_list(channels)
+        q = q.filter(or_(*[Arrival.chan.like(chans) for chans in channels]))
 
     if phases:
-        q = q.filter(Arrival.iphase.in_(phase))
+        phases = make_wildcard_list(phases)
+        q = q.filter(or_(*[Arrival.iphase.like(phase) for phase in phases]))
 
     if t:
         if t.count(None) == 0:
@@ -461,7 +475,8 @@ def get_arrivals(session, arrival, assoc=None, stations=None, channels=None,
         q = q.filter(Assoc.orid.in_(orids))
 
     if auth:
-        q = q.filter(Arrival.auth.in_(auth))
+        auth = make_wildcard_list(auth)
+        q = q.filter(or_(*[Arrival.auth.like(author) for author in auth]))
 
     if asquery:
         res = q
@@ -472,7 +487,7 @@ def get_arrivals(session, arrival, assoc=None, stations=None, channels=None,
 
 
 def get_waveforms(session, wfdisc, station=None, channel=None, starttime=None,
-                  endtime=None, wfids=None, tol=None):
+                  endtime=None, wfids=None, tol=None, asquery=False):
     """
     Request waveforms.
 
@@ -492,6 +507,9 @@ def get_waveforms(session, wfdisc, station=None, channel=None, starttime=None,
     tol : float
         If provided, a warning is fired if any Trace is not within tol seconds
         of starttime and endtime.
+    asquery : bool, optional
+        Return the query object instead of the results.  Default, False.
+        Useful if additional you desire additional sorting of filtering.
 
     Returns
     -------
@@ -503,7 +521,7 @@ def get_waveforms(session, wfdisc, station=None, channel=None, starttime=None,
     # TODO: implement get_arrivals if arrivals=True
     Wfdisc = wfdisc
 
-    st = Stream()
+    # st = Stream()
     if wfids:
         station = channel = starttime = endtime = None
 
@@ -514,9 +532,14 @@ def get_waveforms(session, wfdisc, station=None, channel=None, starttime=None,
     t2_utc = UTCDateTime(endtime) if endtime is not None else None
 
     wfs = get_wfdisc_rows(session, Wfdisc, station, channel, starttime, endtime,
-                          wfids=wfids)
+                          wfids=wfids, asquery=asquery)
 
-    return wfdisc_rows_to_stream(wfs, t1_utc, t2_utc, tol=tol)
+    if asquery:
+        res = wfs
+    else:
+        res = wfdisc_rows_to_stream(wfs, t1_utc, t2_utc, tol=tol)
+
+    return res
     
 
 def wfdisc_rows_to_stream(wf_rows, start_t, end_t, tol=None):
@@ -546,12 +569,14 @@ def wfdisc_rows_to_stream(wf_rows, start_t, end_t, tol=None):
     ValueError:
         Returned Stream contains trace start/end times outside of the tolerance.
     """
-
-    for wf in wfs:
+    st = Stream()
+    
+    for wf in wf_rows:
         try:
             tr = wfdisc2trace(wf)
         except IOError:
             # can't read file
+            # XXX: wow, why the hell would I let unreadable traces slip past
             tr = None
 
         if tr:
@@ -609,3 +634,281 @@ def get_ids(session, lastid, ids, detach=False):
         session.expunge_all(out)
 
     return out
+
+def query_network(session, network, nets=None, affiliation=None, stas=None, time_=None, endtime=None, with_query = None):
+    """
+    Parameters
+    ----------
+    session : sqlalchemy.orm.session instance, bound
+    network : 
+    nets : 
+    affiliation : 
+    stas :
+    time_ :
+    endtime : 
+    pref_nets : 
+    with_query :
+    site_name :  
+    Returns
+    -------
+    query
+        sdfs
+    
+    Notes:
+    ------
+    If input into get_networks is a query, an affiliation table must be provided and the query must contain a site table to be
+    joined on affiliation.sta == site.sta .
+
+    Results cannot be filtered with a station list if affiliation table is not provided.
+    """
+    if with_query:
+        if not affiliation:
+            raise NameError('Affiliation table must be provided when get_networks is given a query as input')   
+
+        q = with_query
+
+        site = None
+        sitechan = None
+        sensor = None
+
+        for i in q.column_descriptions:
+            checkEntity = i['entity']
+            if checkEntity._tabletype == 'Site':
+                site = checkEntity
+            if checkEntity._tabletype == 'Sitechan':
+                sitechan = checkEntity
+            if checkEntity._tabletype == 'Sensor':
+                sensor = checkEntity
+        
+        if site:
+            q = q.add_entity(affiliation)
+            q = q.join(affiliation, affiliation.sta == site.sta)
+            q = q.add_entity(network)
+            q = q.join(network, network.net == affiliation.net)
+
+        elif sitechan:
+            q = q.add_entity(affiliation)
+            q = q.join(affiliation, affiliation.sta == sitechan.sta)
+            q = q.add_entity(network)
+            q = q.join(network, network.net == affiliation.net)
+            warnings.warn("No site table is given in provided query.  Joining Affiliation on Sitechan")
+        
+        elif sensor:
+            q = q.add_entity(affiliation)
+            q = q.join(affiliation, affiliation.sta == sensor.sta)
+            q = q.add_entity(network)
+            q = q.join(network, network.net == affiliation.net)
+            warnings.warn("No Site or Sitechan table is given in provided query.  Joining Affiliation on Sensor")
+
+        else:
+            raise NameError('No table with a sta column on which to join Affiliation is in the provided query')
+
+    else:
+        q = session.query(network)
+        if affiliation:
+            q = q.add_entity(affiliation)
+            q = q.join(affiliation, affiliation.net==network.net)
+
+    if nets:
+        nets = make_wildcard_list(nets)
+        q = q.filter(or_(*[network.net.like(net) for net in nets]))
+
+    if stas:
+        if not affiliation:
+            raise NameError('Affiliation table required to filter Network table from station list')
+        stas = make_wildcard_list(stas)
+        q = q.filter(or_(*[affiliation.sta.like(sta) for sta in stas]))
+
+    if time_:
+        if not affiliation:
+            raise NameError('Affiliation table required to use starttime filtering')
+        q = q.filter(time_.timestamp < affiliation.endtime)
+
+    if endtime:
+        if not affiliation:
+            raise NameError('Affiliation table required to use endtime filtering')
+        q = q.filter(endtime.timestamp > affiliation.time)
+
+    return q
+
+
+def query_site(session, site, sitechan=None, stas=None, chans=None, time_=None, endtime=None, with_query = None):
+    """
+    Parameters
+    ----------
+    session : sqlalchemy.orm.session instance, bound
+    site : 
+    sitechan : 
+    stas : 
+    chans :
+    time_ :
+    endtime :  
+    with_query :
+    affiliation_name :
+    sensor_name:
+
+    Returns
+    -------
+    query
+        sdfs
+    
+    Notes:
+    ------
+    
+    """
+    if with_query:
+
+        q = with_query
+        
+        affiliation = None
+        sensor = None
+
+        for i in q.column_descriptions:
+            checkEntity = i['entity']
+            if checkEntity._tabletype == 'Affiliation':
+                affiliation = checkEntity
+            if checkEntity._tabletype == 'Sensor':
+                sensor = checkEntity
+        
+        if affiliation and sensor:
+            q = q.add_entity(site)
+            q = q.join(site, affiliation.sta == site.sta)
+            if sitechan:
+                q = q.add_entity(sitechan)
+                q = q.join(sitechan, site.sta == sitechan.sta)
+                q = q.join(sitechan, sitechan.chanid == sensor.chanid)
+            else:
+                q = q.join(sensor, site.sta == sensor.sta)
+                warnings.warn("No sitechan specified, joining site to sensor on column sta")
+
+        elif  affiliation:
+            q = q.add_entity(site)
+            q = q.join(site, affiliation.sta == site.sta)
+            if sitechan:
+                q = q.add_entity(sitechan)
+                q = q.join(sitechan, site.sta == sitechan.sta)
+
+        elif sensor:
+            if sitechan:
+                q = q.add_entity(sitechan)
+                q = q.join(sitechan, sitechan.chanid == sensor.chanid)
+                q = q.add_entity(site)
+                q = q.join(site, site.sta == sensor.sta)
+            else:
+                q = q.add_entity(site)
+                q =q.join(site, site.sta == sensor.sta)
+                warnings.warn("No sitechan specified, joining site to sensor on column sta")
+        else: 
+            raise NameError("No affiliation or sensor table in provided in input query for join to site")
+         
+    else:
+        q = session.query(site)
+        if sitechan:
+            q = q.add_entity(sitechan)
+            q = q.join(sitechan, sitechan.sta==site.sta)
+
+    if stas:
+        stas = make_wildcard_list(stas)
+        q = q.filter(or_(*[site.sta.like(sta) for sta in stas]))
+
+    if chans:
+        if not sitechan:
+            raise NameError('Sitechan table required to filter site table by channels')
+        stations = make_wildcard_list(chans)
+        q = q.filter(or_(*[sitechan.chan.like(chan) for chan in chans]))
+
+    if time_:
+        jultime_ = int(time_.strftime('%Y%j'))
+        q = q.filter(jultime_ < site.offdate)
+
+    if endtime:
+        julendtime = int(endtime.strftime('%Y%j'))
+        q = q.filter(julendtime > site.ondate)
+
+    return q
+
+def query_responses(session, sensor, instrument = None, stas = None, chans = None, time_ = None, endtime = None, with_query = None):
+    """
+    Parameters
+    ----------
+    session : sqlalchemy.orm.session instance, bound
+    sensor : 
+    instrument : 
+    stas : 
+    chans :
+    time_ :
+    endtime :  
+    with_query :
+
+    Returns
+    -------
+    query
+        sdfs
+    
+    Notes:
+    ------
+    
+    """
+
+    if with_query:
+        q = with_query
+
+        sitechan = None
+        site = None
+        affiliation = None
+
+        for i in q.column_descriptions:
+            checkEntity = i['entity']
+            if checkEntity._tabletype == 'Sitechan':
+                sitechan = checkEntity
+            if checkEntity._tabletype == 'Site':
+                site = checkEntity
+            if checkEntity._tabletype == 'Affiliation':
+                affiliation = checkEntity
+        
+        if sitechan:
+            q = q.add_entity(sensor)
+            q = q.join(sensor, sitechan.chanid == sensor.chanid)
+
+        elif site:
+            q = q.add_entity(sensor)
+            q = q.join(sensor, sensor.sta == site.sta)
+            warnings.warn("No sitechan specified, joining sensor to site on column sta")
+        
+        elif affiliation:
+            q = q.add_entity(sensor)
+            q = q.join(sensor, sensor.sta == affiliation.sta)
+            warnings.warn("No site or sitechan specified, joining sensor to affiliation on column sta")
+        
+        else:
+            raise NameError("No table in provided query on which to join the sensor table ")
+
+    else:
+        q = session.query(sensor)
+    
+    if instrument:
+        q = q.add_entity(instrument)
+        q = q.join(instrument, sensor.inid == instrument.inid)
+
+    if stas:
+        stas = make_wildcard_list(stas)
+        q = q.filter(or_(*[sensor.sta.like(sta) for sta in stas]))
+
+    if chans:
+        chans = make_wildcard_list(chans)
+        q = q.filter(or_(*[sensor.chan.like(chan) for chan in chans]))
+
+    if time_:
+        q = q.filter(time_.timestamp < sensor.endtime)
+
+    if endtime:
+        q = q.filter(endtime.timestamp > sensor.time)
+
+
+    return q
+
+def assign_unique_net(q, network_name, affiliation_name, pref_nets = None, two_char_code = True, first_available = True, default_net = '__'):
+    return 
+
+def check_orphan_stas():
+    return
