@@ -37,6 +37,7 @@ Define a new abstract class in this schema.
 >>> import sqlalchemy as sa
 >>> from sqlalchemy.ext.declarative import declared_attr
 >>> import pisces.schema.kbcore as kb
+>>> from copy import deepcopy as dc
 >>>
 >>> def strparse(s):
 >>>     return s.strip()
@@ -48,16 +49,16 @@ Define a new abstract class in this schema.
 >>>         return (sa.PrimaryKeyConstraint('wfid'),)
 >>>
 >>>     wfid = kb.wfid._copy()
->>>     azimuth = kb.azimuth._copy()
+>>>     azimuth = dc(kb.azimuth)
 >>>     slo = sa.Column(Float(24),
 >>>         info={'default': -1.0, 'parse': float, 'width': 7, 'format': '7.4f'})
 >>>     filter = sa.Column(String(30),
 >>>         info={'default': '-', 'parse': strparse, 'width': 30, 'format': '30.30s'})
 >>>     recipe = sa.Column(String(15),
 >>>         info={'default': '-', 'parse': strparse, 'width': 15, 'format': '15.15s'})
->>>     algorithm = kb.algorithm._copy()
->>>     auth = kb.auth._copy()
->>>     lddate = kb.lddate._copy()
+>>>     algorithm = dc(kb.algorithm)
+>>>     auth = dc(kb.auth)
+>>>     lddate = dc(kb.lddate)
 
 """
 from datetime import datetime
@@ -71,6 +72,7 @@ from sqlalchemy.ext.declarative import declared_attr
 from pisces.schema.util import PiscesMeta
 from pisces.schema.util import parse_str, parse_int, parse_float
 from pisces.io.trace import wfdisc2trace
+from copy import deepcopy as dc
 
 # TODO: check __table_args__ syntax for final empty {}
 # TODO: use http://docs.sqlalchemy.org/en/rel_0_8/orm/extensions/hybrid.html?
@@ -89,7 +91,7 @@ Base = declarative_base(metaclass=PiscesMeta, constructor=None)
 #       http://stackoverflow.com/questions/16928644/floats-in-numpy-structured-array-and-native-string-formatting-with-format
 # NOTE: column names are not defined here, they are assigned during declarative
 #       table definitions.  i.e. amp.name is None until it is used later.  This
-#       allows recycling columns as different names, like sta1 = sta._copy().
+#       allows recycling columns as different names, like sta1 = dc(sta1)).
 
 # TODO: write a parser that gets the width (and type?) from info['format']
 
@@ -540,11 +542,11 @@ class Affiliation(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('net', 'sta', 'time'),)
 
-    net = net._copy()
-    sta = sta._copy()
-    time = time._copy()
-    endtime = endtime._copy()
-    lddate = lddate._copy()
+    net = dc(net)
+    sta = dc(sta)
+    time = dc(time)
+    endtime = dc(endtime)
+    lddate = dc(lddate)
 
 
 class Amplitude(Base):
@@ -554,23 +556,23 @@ class Amplitude(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('ampid'),)
 
-    ampid = ampid._copy()
-    arid = arid._copy()
-    parid = parid._copy()
-    chan = chan._copy()
-    amp = amp._copy()
-    per = per._copy()
-    snr = snr._copy()
-    amptime = amptime._copy()
-    time = time._copy()
-    duration = duration._copy()
-    deltaf = deltaf._copy()
-    amptype = amptype._copy()
-    units = units._copy()
-    clip = clip._copy()
-    inarrival = inarrival._copy()
-    auth = auth._copy()
-    lddate = lddate._copy()
+    ampid = dc(ampid)
+    arid = dc(arid)
+    parid = dc(parid)
+    chan = dc(chan)
+    amp = dc(amp)
+    per = dc(per)
+    snr = dc(snr)
+    amptime = dc(amptime)
+    time = dc(time)
+    duration = dc(duration)
+    deltaf = dc(deltaf)
+    amptype = dc(amptype)
+    units = dc(units)
+    clip = dc(clip)
+    inarrival = dc(inarrival)
+    auth = dc(auth)
+    lddate = dc(lddate)
 
 
 class Arrival(Base):
@@ -581,32 +583,32 @@ class Arrival(Base):
         return (PrimaryKeyConstraint('arid'),
                 UniqueConstraint('sta', 'time', 'chan', 'iphase', 'auth'),)
 
-    sta = sta._copy()
-    time = time._copy()
-    arid = arid._copy()
-    jdate = jdate._copy()
-    stassid = stassid._copy()
-    chanid = chanid._copy()
-    chan = chan._copy()
-    iphase = iphase._copy()
-    stype = stype._copy()
-    deltim = deltim._copy()
-    azimuth = azimuth._copy()
-    delaz = delaz._copy()
-    slow = slow._copy()
-    delslo = delslo._copy()
-    ema = ema._copy()
-    rect = rect._copy()
-    amp = amp._copy()
-    per = per._copy()
-    logat = logat._copy()
-    clip = clip._copy()
-    fm = fm._copy()
-    snr = snr._copy()
-    qual = qual._copy()
-    auth = auth._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    sta = dc(sta)
+    time = dc(time)
+    arid = dc(arid)
+    jdate = dc(jdate)
+    stassid = dc(stassid)
+    chanid = dc(chanid)
+    chan = dc(chan)
+    iphase = dc(iphase)
+    stype = dc(stype)
+    deltim = dc(deltim)
+    azimuth = dc(azimuth)
+    delaz = dc(delaz)
+    slow = dc(slow)
+    delslo = dc(delslo)
+    ema = dc(ema)
+    rect = dc(rect)
+    amp = dc(amp)
+    per = dc(per)
+    logat = dc(logat)
+    clip = dc(clip)
+    fm = dc(fm)
+    snr = dc(snr)
+    qual = dc(qual)
+    auth = dc(auth)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Assoc(Base):
@@ -616,25 +618,25 @@ class Assoc(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('arid', 'orid'), UniqueConstraint('arid'),)
 
-    arid = arid._copy()
-    orid = orid._copy()
-    sta = sta._copy()
-    phase = phase._copy()
-    belief = belief._copy()
-    delta = delta._copy()
-    seaz = seaz._copy()
-    esaz = esaz._copy()
-    timeres = timeres._copy()
-    timedef = timedef._copy()
-    azres = azres._copy()
-    azdef = azdef._copy()
-    slores = slores._copy()
-    slodef = slodef._copy()
-    emares = emares._copy()
-    wgt = wgt._copy()
-    vmodel = vmodel._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    arid = dc(arid)
+    orid = dc(orid)
+    sta = dc(sta)
+    phase = dc(phase)
+    belief = dc(belief)
+    delta = dc(delta)
+    seaz = dc(seaz)
+    esaz = dc(esaz)
+    timeres = dc(timeres)
+    timedef = dc(timedef)
+    azres = dc(azres)
+    azdef = dc(azdef)
+    slores = dc(slores)
+    slodef = dc(slodef)
+    emares = dc(emares)
+    wgt = dc(wgt)
+    vmodel = dc(vmodel)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Event(Base):
@@ -644,12 +646,12 @@ class Event(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('evid'), UniqueConstraint('prefor'),)
     
-    evid = evid._copy()
-    evname = evname._copy()
-    prefor = prefor._copy()
-    auth = auth._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    evid = dc(evid)
+    evname = dc(evname)
+    prefor = dc(prefor)
+    auth = dc(auth)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Gregion(Base):
@@ -659,9 +661,9 @@ class Gregion(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('grn'),)
 
-    grn = grn._copy()
-    grname = grname._copy()
-    lddate = lddate._copy()
+    grn = dc(grn)
+    grname = dc(grname)
+    lddate = dc(lddate)
 
 
 class Instrument(Base):
@@ -671,18 +673,18 @@ class Instrument(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('inid'),)
 
-    inid = inid._copy()
-    insname = insname._copy()
-    instype = instype._copy()
-    band = band._copy()
-    digital = digital._copy()
-    samprate = samprate._copy()
-    ncalib = ncalib._copy()
-    ncalper = ncalper._copy()
-    dir = dir._copy()
-    dfile = dfile._copy()
-    rsptype = rsptype._copy()
-    lddate = lddate._copy()
+    inid = dc(inid)
+    insname = dc(insname)
+    instype = dc(instype)
+    band = dc(band)
+    digital = dc(digital)
+    samprate = dc(samprate)
+    ncalib = dc(ncalib)
+    ncalper = dc(ncalper)
+    dir = dc(dir)
+    dfile = dc(dfile)
+    rsptype = dc(rsptype)
+    lddate = dc(lddate)
 
 
 # TODO: implement class-wide generator expressions on keyvalues.
@@ -705,9 +707,9 @@ class Lastid(Base):
         self.keyvalue += 1
         return self.keyvalue
 
-    keyname = keyname._copy()
-    keyvalue = keyvalue._copy()
-    lddate = lddate._copy()
+    keyname = dc(keyname)
+    keyvalue = dc(keyvalue)
+    lddate = dc(lddate)
 
 
 class Netmag(Base):
@@ -718,17 +720,17 @@ class Netmag(Base):
         return (PrimaryKeyConstraint('magid'),
                 UniqueConstraint('magid', 'orid'),)
 
-    magid = magid._copy()
-    net = net._copy()
-    orid = orid._copy()
-    evid = evid._copy()
-    magtype = magtype._copy()
-    nsta = nsta._copy()
-    magnitude = magnitude._copy()
-    uncertainty = uncertainty._copy()
-    auth = auth._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    magid = dc(magid)
+    net = dc(net)
+    orid = dc(orid)
+    evid = dc(evid)
+    magtype = dc(magtype)
+    nsta = dc(nsta)
+    magnitude = dc(magnitude)
+    uncertainty = dc(uncertainty)
+    auth = dc(auth)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Network(Base):
@@ -738,12 +740,12 @@ class Network(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('net'),)
 
-    net = net._copy()
-    netname = netname._copy()
-    nettype = nettype._copy()
-    auth = auth._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    net = dc(net)
+    netname = dc(netname)
+    nettype = dc(nettype)
+    auth = dc(auth)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Origerr(Base):
@@ -753,26 +755,26 @@ class Origerr(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('orid'),)
 
-    orid = orid._copy()
-    sxx = sxx._copy()
-    syy = syy._copy()
-    szz = szz._copy()
-    stt = stt._copy()
-    sxy = sxy._copy()
-    sxz = sxz._copy()
-    syz = syz._copy()
-    stx = stx._copy()
-    sty = sty._copy()
-    stz = stz._copy()
-    sdobs = sdobs._copy()
-    smajax = smajax._copy()
-    sminax = sminax._copy()
-    strike = strike._copy()
-    sdepth = sdepth._copy()
-    stime = stime._copy()
-    conf = conf._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    orid = dc(orid)
+    sxx = dc(sxx)
+    syy = dc(syy)
+    szz = dc(szz)
+    stt = dc(stt)
+    sxy = dc(sxy)
+    sxz = dc(sxz)
+    syz = dc(syz)
+    stx = dc(stx)
+    sty = dc(sty)
+    stz = dc(stz)
+    sdobs = dc(sdobs)
+    smajax = dc(smajax)
+    sminax = dc(sminax)
+    strike = dc(strike)
+    sdepth = dc(sdepth)
+    stime = dc(stime)
+    conf = dc(conf)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Origin(Base):
@@ -782,31 +784,31 @@ class Origin(Base):
     def __table_args__(cls):
         return (UniqueConstraint('lat', 'lon', 'depth', 'time', 'auth'), PrimaryKeyConstraint('orid'),)
 
-    lat = lat._copy()
-    lon = lon._copy()
-    depth = depth._copy()
-    time = time._copy()
-    orid = orid._copy()
-    evid = evid._copy()
-    jdate = jdate._copy()
-    nass = nass._copy()
-    ndef = ndef._copy()
-    ndp = ndp._copy()
-    grn = grn._copy()
-    srn = srn._copy()
-    etype = etype._copy()
-    depdp = depdp._copy()
-    dtype = dtype._copy()
-    mb = mb._copy()
-    mbid = mbid._copy()
-    ms = ms._copy()
-    msid = msid._copy()
-    ml = ml._copy()
-    mlid = mlid._copy()
-    algorithm = algorithm._copy()
-    auth = auth._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    lat = dc(lat)
+    lon = dc(lon)
+    depth = dc(depth)
+    time = dc(time)
+    orid = dc(orid)
+    evid = dc(evid)
+    jdate = dc(jdate)
+    nass = dc(nass)
+    ndef = dc(ndef)
+    ndp = dc(ndp)
+    grn = dc(grn)
+    srn = dc(srn)
+    etype = dc(etype)
+    depdp = dc(depdp)
+    dtype = dc(dtype)
+    mb = dc(mb)
+    mbid = dc(mbid)
+    ms = dc(ms)
+    msid = dc(msid)
+    ml = dc(ml)
+    mlid = dc(mlid)
+    algorithm = dc(algorithm)
+    auth = dc(auth)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Remark(Base):
@@ -816,10 +818,10 @@ class Remark(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('commid', 'lineno'),)
 
-    commid = commid._copy()
-    lineno = lineno._copy()
-    remark = remark._copy()
-    lddate = lddate._copy()
+    commid = dc(commid)
+    lineno = dc(lineno)
+    remark = dc(remark)
+    lddate = dc(lddate)
 
 
 class Sensor(Base):
@@ -829,18 +831,18 @@ class Sensor(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('sta', 'chan', 'time', 'endtime'),)
 
-    sta = sta._copy()
-    chan = chan._copy()
-    time = time._copy()
-    endtime = endtime._copy()
-    inid = inid._copy()
-    chanid = chanid._copy()
-    jdate = jdate._copy()
-    calratio = calratio._copy()
-    calper = calper._copy()
-    tshift = tshift._copy()
-    instant = instant._copy()
-    lddate = lddate._copy()
+    sta = dc(sta)
+    chan = dc(chan)
+    time = dc(time)
+    endtime = dc(endtime)
+    inid = dc(inid)
+    chanid = dc(chanid)
+    jdate = dc(jdate)
+    calratio = dc(calratio)
+    calper = dc(calper)
+    tshift = dc(tshift)
+    instant = dc(instant)
+    lddate = dc(lddate)
 
 
 class Site(Base):
@@ -850,18 +852,18 @@ class Site(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('sta', 'ondate'),)
 
-    sta = sta._copy()
-    ondate = ondate._copy()
-    offdate = offdate._copy()
-    lat = lat._copy()
-    lon = lon._copy()
-    elev = elev._copy()
-    staname = staname._copy()
-    statype = statype._copy()
-    refsta = refsta._copy()
-    dnorth = dnorth._copy()
-    deast = deast._copy()
-    lddate = lddate._copy()
+    sta = dc(sta)
+    ondate = dc(ondate)
+    offdate = dc(offdate)
+    lat = dc(lat)
+    lon = dc(lon)
+    elev = dc(elev)
+    staname = dc(staname)
+    statype = dc(statype)
+    refsta = dc(refsta)
+    dnorth = dc(dnorth)
+    deast = dc(deast)
+    lddate = dc(lddate)
 
 
 class Sitechan(Base):
@@ -872,17 +874,17 @@ class Sitechan(Base):
         return (UniqueConstraint('sta', 'chan', 'ondate'),
                 PrimaryKeyConstraint('chanid'))
 
-    sta = sta._copy()
-    chan = chan._copy()
-    ondate = ondate._copy()
-    chanid = chanid._copy()
-    offdate = offdate._copy()
-    ctype = ctype._copy()
-    edepth = edepth._copy()
-    hang = hang._copy()
-    vang = vang._copy()
-    descrip = descrip._copy()
-    lddate = lddate._copy()
+    sta = dc(sta)
+    chan = dc(chan)
+    ondate = dc(ondate)
+    chanid = dc(chanid)
+    offdate = dc(offdate)
+    ctype = dc(ctype)
+    edepth = dc(edepth)
+    hang = dc(hang)
+    vang = dc(vang)
+    descrip = dc(descrip)
+    lddate = dc(lddate)
 
 
 class Sregion(Base):
@@ -892,9 +894,9 @@ class Sregion(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('srn'),)
 
-    srn = srn._copy()
-    srname = srname._copy()
-    lddate = lddate._copy()
+    srn = dc(srn)
+    srname = dc(srname)
+    lddate = dc(lddate)
 
 
 class Stamag(Base):
@@ -904,23 +906,23 @@ class Stamag(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('magid', 'sta', 'arid'),)
 
-    magid = magid._copy()
-    ampid = ampid._copy()
-    sta = sta._copy()
-    arid = arid._copy()
-    orid = orid._copy()
-    evid = evid._copy()
-    phase = phase._copy()
-    delta = delta._copy()
-    magtype = magtype._copy()
-    magnitude = magnitude._copy()
-    uncertainty = uncertainty._copy()
-    magres = magres._copy()
-    magdef = magdef._copy()
-    mmodel = mmodel._copy()
-    auth = auth._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    magid = dc(magid)
+    ampid = dc(ampid)
+    sta = dc(sta)
+    arid = dc(arid)
+    orid = dc(orid)
+    evid = dc(evid)
+    phase = dc(phase)
+    delta = dc(delta)
+    magtype = dc(magtype)
+    magnitude = dc(magnitude)
+    uncertainty = dc(uncertainty)
+    magres = dc(magres)
+    magdef = dc(magdef)
+    mmodel = dc(mmodel)
+    auth = dc(auth)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Wfdisc(Base):
@@ -941,26 +943,26 @@ class Wfdisc(Base):
         """
         return wfdisc2trace(self)
 
-    sta = sta._copy()
-    chan = chan._copy()
-    time = time._copy()
-    wfid = wfid._copy()
-    chanid = chanid._copy()
-    jdate = jdate._copy()
-    endtime = endtime._copy()
-    nsamp = nsamp._copy()
-    samprate = samprate._copy()
-    calib = calib._copy()
-    calper = calper._copy()
-    instype = instype._copy()
-    segtype = segtype._copy()
-    datatype = datatype._copy()
-    clip = clip._copy()
-    dir = dir._copy()
-    dfile = dfile._copy()
-    foff = foff._copy()
-    commid = commid._copy()
-    lddate = lddate._copy()
+    sta = dc(sta)
+    chan = dc(chan)
+    time = dc(time)
+    wfid = dc(wfid)
+    chanid = dc(chanid)
+    jdate = dc(jdate)
+    endtime = dc(endtime)
+    nsamp = dc(nsamp)
+    samprate = dc(samprate)
+    calib = dc(calib)
+    calper = dc(calper)
+    instype = dc(instype)
+    segtype = dc(segtype)
+    datatype = dc(datatype)
+    clip = dc(clip)
+    dir = dc(dir)
+    dfile = dc(dfile)
+    foff = dc(foff)
+    commid = dc(commid)
+    lddate = dc(lddate)
 
 
 class Wftag(Base):
@@ -970,10 +972,10 @@ class Wftag(Base):
     def __table_args__(cls):
         return (PrimaryKeyConstraint('tagname', 'tagid', 'wfid'),)
 
-    tagname = tagname._copy()
-    tagid = tagid._copy()
-    wfid = wfid._copy()
-    lddate = lddate._copy()
+    tagname = dc(tagname)
+    tagid = dc(tagid)
+    wfid = dc(wfid)
+    lddate = dc(lddate)
 
 
 # possibly useful indexes
